package javax.imageio.stream;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.attribute.FileAttribute;
import com.sun.imageio.stream.StreamCloser;
import java.io.RandomAccessFile;
import java.io.File;
import java.io.OutputStream;

public class FileCacheImageOutputStream extends ImageOutputStreamImpl
{
    private OutputStream stream;
    private File cacheFile;
    private RandomAccessFile cache;
    private long maxStreamPos;
    private final StreamCloser.CloseAction closeAction;
    
    public FileCacheImageOutputStream(OutputStream stream, File file) throws IOException {
        this.maxStreamPos = 0L;
        if (stream == null) {
            throw new IllegalArgumentException("stream == null!");
        }
        if (file != null && !file.isDirectory()) {
            throw new IllegalArgumentException("Not a directory!");
        }
        this.stream = stream;
        if (file == null) {
            this.cacheFile = Files.createTempFile("imageio", ".tmp", new FileAttribute[0]).toFile();
        }
        else {
            this.cacheFile = Files.createTempFile(file.toPath(), "imageio", ".tmp", new FileAttribute[0]).toFile();
        }
        this.cache = new RandomAccessFile(this.cacheFile, "rw");
        StreamCloser.addToQueue(this.closeAction = StreamCloser.createCloseAction(this));
        return;
    }
    
    @Override
    public int read() throws IOException {
        this.checkClosed();
        this.bitOffset = 0;
        int read = this.cache.read();
        if (read != -1) {
            ++this.streamPos;
        }
        return read;
    }
    
    @Override
    public int read(byte[] b, int off, int len) throws IOException {
        this.checkClosed();
        if (b == null) {
            throw new NullPointerException("b == null!");
        }
        if (off < 0 || len < 0 || off + len > b.length || off + len < 0) {
            throw new IndexOutOfBoundsException("off < 0 || len < 0 || off+len > b.length || off+len < 0!");
        }
        this.bitOffset = 0;
        if (len == 0) {
            return 0;
        }
        int read = this.cache.read(b, off, len);
        if (read != -1) {
            this.streamPos += (long)read;
        }
        return read;
    }
    
    @Override
    public void write(int b) throws IOException {
        this.flushBits();
        this.cache.write(b);
        ++this.streamPos;
        this.maxStreamPos = Math.max(this.maxStreamPos, this.streamPos);
    }
    
    @Override
    public void write(byte[] b, int off, int len) throws IOException {
        this.flushBits();
        this.cache.write(b, off, len);
        this.streamPos += (long)len;
        this.maxStreamPos = Math.max(this.maxStreamPos, this.streamPos);
    }
    
    @Override
    public long length() {
        try {
            this.checkClosed();
            return this.cache.length();
        }
        catch (IOException ex) {
            return -1L;
        }
    }
    
    @Override
    public void seek(long pos) throws IOException {
        this.checkClosed();
        if (pos < this.flushedPos) {
            throw new IndexOutOfBoundsException();
        }
        this.cache.seek(pos);
        this.streamPos = this.cache.getFilePointer();
        this.maxStreamPos = Math.max(this.maxStreamPos, this.streamPos);
        this.bitOffset = 0;
        return;
    }
    
    @Override
    public boolean isCached() {
        return true;
    }
    
    @Override
    public boolean isCachedFile() {
        return true;
    }
    
    @Override
    public boolean isCachedMemory() {
        return false;
    }
    
    @Override
    public void close() throws IOException {
        this.seek(this.maxStreamPos = this.cache.length());
        this.flushBefore(this.maxStreamPos);
        super.close();
        this.cache.close();
        this.cache = null;
        this.cacheFile.delete();
        this.cacheFile = null;
        this.stream.flush();
        this.stream = null;
        StreamCloser.removeFromQueue(this.closeAction);
    }
    
    @Override
    public void flushBefore(long n) throws IOException {
        long flushedPos = this.flushedPos;
        super.flushBefore(n);
        long a = this.flushedPos - flushedPos;
        if (a > 0L) {
            int n2 = 512;
            byte[] array = new byte[n2];
            this.cache.seek(flushedPos);
            while (a > 0L) {
                int n3 = (int)Math.min(a, (long)n2);
                this.cache.readFully(array, 0, n3);
                this.stream.write(array, 0, n3);
                a -= (long)n3;
            }
            this.stream.flush();
        }
    }
}
